/*
 * Code: Ben Walding
 * Date: 1 January 1998
 *
 * Not that this is a very nice implementation. Since cdlabel knows about this files' 
 * structures.
 * Protected by the GNU Public License v2.0  <distrib>/GPL.LICENSE 
 */


#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>

#include "workmandb.h"
#include "cdlabel.h"
#include "database.h"


FILE *workman_open_database(char *workmandb);
void  workman_read_database(FILE *database);
void  workman_close_database(FILE *database);


/*
 * Load the WorkMan database
 */
void workman_load_database(char *workmandb) {
  FILE *database;

  /* Display errors - XXX XXX XXX - Needs a good method compatible with CDDB etc*/
  if (!(database = workman_open_database(workmandb))) {
    return;
  }

  workman_read_database(database);
  workman_close_database(database);
}



FILE *workman_open_database(char *workmandb) {
  char *pref=getenv("WORKMANDB");
  char *home=getenv("HOME") ? getenv("HOME"):"";/* XXX XXX XXX - just stupid */
  char infile[1024];
  FILE *database;
  
  if (workmandb) {
    strcpy(infile, workmandb);
  } else if (pref) {
    strcpy(infile, pref);
  } else {
    strcpy(infile,home);
    strcat(infile,"/.workmandb");
  }

  if (!(database = fopen(infile, "rt"))) {
    perror(infile);
  }

  return database;
}


// Needs rewriting so it is clear
void workman_read_database(FILE *database) {
  Album *current_album = NULL;
  char line[1024];
  char trackstr[1024];
  int lineNum = 0;
  char *buf = NULL; /* The compiler complains, but there is no error due to how the algorithm works. */
  
  while (!feof(database)) {
	char key[64];
	char value[1024];
	char *pos;
	
	fgets(line, sizeof(line), database);
	lineNum++;
	
	/* Simplistic comment ignoring XXX XXX XXX */
	if (line[0] == '#')
	  continue;
	
	pos = strstr(line, " "); /* Could be changed to inline loop XXX XXX XXX - why bother?*/
	
	/* If the line doesn't take the form of "key value" (no spaces), ignore it */
	if (!pos)
	  continue;
	
	*pos = '\0';  //Split the string into key and value
	pos++;
	strcpy(key, line);  // Move key into easier to handle storage
	strcpy(value, pos); // Move value ...
	value[strlen(value) - 1] = '\0';    // Remove the trailing \n
	

	// tracks <albumlength> <track1 start> <track2 start> ...
	if (!strcmp(key, "tracks")) {

	  //If this isn't the first album, there remains some processing to be done on the last one
	  if (current_album != NULL) {
		Track *track_ptr = current_album->tracks;
		current_album->len = atoi(buf);
		
		if (track_ptr) {
		  for (; track_ptr->next; track_ptr = track_ptr->next) 
			track_ptr->len = (track_ptr->next->addr - track_ptr->addr) / 75;
		  track_ptr->len = current_album->len - track_ptr->addr / 75;
		}
	  }

	  //Allocate and install new album memory
	  current_album = new Album();
	  
	  //Store the tracks string... this is needed, because it isn't actually handled until each of the
	  //Track titles is read in
	  strcpy(trackstr, value);
	  current_album->ntracks = atoi(trackstr);
	  
	  buf = strstr(trackstr, " ");
	  buf++;
	  continue;
	}
	
	/* This is bad, error in workmandb file */
	if (!current_album) {
	  continue;
	}
	
	if (!strcmp(key, "cdname")) {
	  // printf("Key: %s Value: %s\n", key, value);
	  current_album->setTitle(value);
	  continue;
	}
    
	if (!strcmp(key, "artist")) {
	  current_album->setArtist(value);
	  continue;
	}
	
	if (!strcmp(key, "track")) {
	  Track *track_ptr = new Track(current_album);
	  track_ptr->setTitle(value);
	  track_ptr->addr = atoi(buf);
	  
	  buf = strstr(buf, " ");
	  buf++;
	  continue;
	}
	// Haven't processed this value yet
	printf("Line: %d     Error unknown Key: %s   value: %s\n", lineNum, key, value);
	
  }
  
  //Process last
  if (current_album != NULL) {
	Track *track_ptr = current_album->tracks;
	current_album->len = atoi(buf);
	
	if (track_ptr) {
	  for (; track_ptr->next; track_ptr = track_ptr->next) {
		track_ptr->len = (track_ptr->next->addr - track_ptr->addr) / 75;
	  }
	  track_ptr->len = current_album->len - track_ptr->addr / 75;
	}
  }
  return;
}


void workman_close_database(FILE *database) {
  fclose(database);
  return;
}

